"use client";

import { z } from "zod";
import Link from "next/link";
import {
  Card,
  CardContent,
  CardDescription,
  CardHeader,
  CardTitle,
} from "@/components/ui/card";
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from "@/components/ui/form";
import logger from "@/lib/logger";
import { useEffect } from "react";
import { Loader2 } from "lucide-react";
import { useForm } from "react-hook-form";
import { useToast } from "@/hooks/use-toast";
import { Input } from "@/components/ui/input";
import { Button } from "@/components/ui/button";
import { ToastAction } from "@/components/ui/toast";
import { zodResolver } from "@hookform/resolvers/zod";
import userGetCurrentStore from "@/app/actions/(gitea)/user/store";

const formSchema = z.object({
  username: z.string().refine(
    (value) => {
      const isEmail = value.includes("@");
      if (isEmail) {
        return z
          .string()
          .email("Invalid email address")
          .max(254, "Email is too long (max 254 characters)")
          .safeParse(value).success;
      } else {
        return z
          .string()
          .min(1, "Username cannot be empty")
          .max(40, "Username is too long (max 40 characters)")
          .safeParse(value).success;
      }
    },
    {
      message: "Invalid email address or username",
    }
  ),
  password: z
    .string()
    .min(8, "Password must be at least 8 characters")
    .max(255, "Password must be at most 255 characters"),
});

const defaultValues: z.infer<typeof formSchema> = {
  username: "",
  password: "",
};

export function LoginForm() {
  const { toast } = useToast();
  const { user, error, loading, userGetCurrent } = userGetCurrentStore();

  const form = useForm<z.infer<typeof formSchema>>({
    resolver: zodResolver(formSchema),
    defaultValues,
  });

  function onSubmit(values: z.infer<typeof formSchema>) {
    logger.info({ values }, "submitting form");
    userGetCurrent(values.username, values.password).then(() => {
      form.reset(defaultValues);
    });
  }

  function onReset() {
    form.reset(defaultValues);
  }

  useEffect(() => {
    form.reset(defaultValues);
  }, [form]);

  useEffect(() => {
    if (user) {
      toast({
        variant: "default",
        title: "Successfully Logged In",
        description: `Welcome back, ${user.login}`,
        action: <ToastAction altText="Continue">Continue</ToastAction>,
      });
    } else if (error) {
      toast({
        variant: "destructive",
        title: "Error Logging In",
        description: error.message,
        action: <ToastAction altText="Retry">Retry</ToastAction>,
      });
    }
  }, [user, error, toast]);

  return (
    <Card className="mx-auto max-w-sm">
      <CardHeader>
        <CardTitle className="text-2xl">Login</CardTitle>
        <CardDescription>
          Enter your email below to login to your account
        </CardDescription>
      </CardHeader>
      <CardContent>
        <Form {...form}>
          <form onSubmit={form.handleSubmit(onSubmit)} className="grid gap-4">
            <FormField
              control={form.control}
              name="username"
              render={({ field }) => (
                <FormItem className="grid gap-2">
                  <FormLabel>Username or Email Address</FormLabel>
                  <FormControl>
                    <Input {...field} />
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />
            <FormField
              control={form.control}
              name="password"
              render={({ field }) => (
                <FormItem className="grid gap-2">
                  <div className="flex items-center">
                    <FormLabel>Password</FormLabel>
                    <Link
                      href="#"
                      className="ml-auto inline-block text-sm underline"
                    >
                      Forgot your password?
                    </Link>
                  </div>
                  <FormControl>
                    <Input {...field} type="password" />
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />
            <div className="flex items-center justify-between">
              <Button variant="outline" onClick={onReset}>
                Reset
              </Button>
              <Button type="submit" disabled={loading}>
                {loading ? (
                  <>
                    <Loader2 className="animate-spin" /> Login...
                  </>
                ) : (
                  "Login"
                )}
              </Button>
            </div>
          </form>
        </Form>
      </CardContent>
    </Card>
  );
}
